package Reporting;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(debug_report failed_report ok_report start_report command_report initial_report report_and_die);

use strict;

# uses values set in the configuration section ($log_to_njs, main::CMD_SOCK, $debug)

#
# Logging subs.
#
# Destination of log entries can be either stdout or
# the NJS (plus syslog before NJS connections made)
#
# More verbose messaging if "debug" set
#

# Write a string to the current output with a time stamp (internal)
#
# arg1: string to write

sub report {

    my $time = scalar localtime;
    my $string = shift;

    $string =~ s/\s+$//;

    if($main::log_to_njs) {
	print main::CMD_SOCK "TSI_COMMENT\n$time $string\nEND_TSI_COMMENT\n";
    }
    else {
        print "$time $string\n";
    }

}

#
# Write debug information to current output (if debug requested)
#
# arg1: string to write
#
sub debug_report {
    if($main::debug) {
        my $string = shift;
        report("     $string");
    }
}

#
# Execution of an NJS request failed, tell NJS with appropriate protocol
# and also write the message to current output
#
# arg1: failure message
#
sub failed_report {

    my $string = shift;

    # NJS expects a single line, remove any embedded new lines
    $string =~ s/\n/:/g;

    report(" ### Command execution FAILED");
    report("     $string");
    report("     Command was: $Reporting::reported_command") if $Reporting::reported_command;
    report(" ### End report command execution FAILED");

    # Tell NJS
    print main::CMD_SOCK "TSI_FAILED: $string\n";

}

#
# Execution of an NJS request executed OK, tell NJS with appropriate protocol,
# and also write the message to current output (if debugging)
#
# arg1: debug message 
#
sub ok_report {

    my $string = shift;

    report(" --- Command executed OK");
    debug_report("$string");

    # Tell NJS
    print main::CMD_SOCK "TSI_OK\n";

}

#
# Start of execution of a request from NJS, write to current output
#
# arg1: message 
#
sub start_report {
    my $string = shift;
    report(" --- $string");
}

#
# This is the command that was executed, report if debugging
# record for possible printing by a failed command
#
# arg1: the executed command
#
sub command_report {
    $Reporting::reported_command = shift;  # keep this for use by failed_report
    debug_report("Command is: $Reporting::reported_command");
}

#
# Information to log at the start of execution, before
# a connection is made to NJS so to syslog (one day) or stdout.
# (cannot go to NJS because we have not contacted one yet)
#
# arg1: message to write
#
sub initial_report {

    my $string = shift;
    my $time = scalar localtime;

	print "$time     $string\n";    # should be to a system log?
}

#
# Something has happened to make this execution
# die (assumed that NJS has gone so will log to
# syslog). Should not ever be called.
#
# arg1: message to write
#
sub report_and_die {
# Cannot report, this is called by children with no stdout/stderr
    my $string = shift;
    initial_report("UNICORE TSI $$: $string");
    exit 1;
}

# $Id: Reporting.pm,v 1.17 2002/04/09 17:42:10 svdb Exp $
#
#
# 12-19-02 Klaus-Dieter Oertel Pallas take care of $main::log_to_njs again
# 07-10-01 Sven van den Berghe fecit Ignores $main::log_to_njs (for new TSID execution model)
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
#
#                  Copyright (c) Fujitsu Ltd 2000 - 2002
#
#     Use and distribution is subject to the Community Source Licence.
#                                                       
# Further information is available from www.unicore.org or arcon@fecit.co.uk.
#
