package com.fujitsu.arcon.gateway;

import com.fujitsu.arcon.gateway.logger.*;

import org.unicore.Vsite;

import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;

import java.io.*;

import java.util.StringTokenizer;

import com.sun.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;

/**
 * A factory for connections that pass requests (fileTransferRequest) to a AFT
 * running inside the NJS (this is a minimal implementation).
 *
 * @author Sven van den Berghe, fujitsu
 *
 * @version $Id: AFTVsiteConnectionFactory.java,v 1.4 2003/02/10 18:25:47 svdb Exp $
 *
 **/

public class AFTVsiteConnectionFactory implements VsiteConnectionFactory {

    /**
     * Give the NJS class loader something to call, complete
     * initialisation is through init().
     *
     **/
    public AFTVsiteConnectionFactory() {
    }

    private String vsite_name;

    public String getName() {return vsite_name;}

    private InetAddress host_address;
    private int host_port;

    private Logger logger;

    /**
     * Initialise the instance using a definition line from the Vsites file.
     * <p>
     * Expect the init string to contain the NJS host name and  NJS port number
     *
     * @return Success of initialisation
     *
     **/
    public boolean init(Vsite vsite, String init_string) {

        logger = LoggerManager.get("com.fujitsu.arcon.gateway.VsiteConnectionFactoryImpl");

        vsite_name = vsite.getName();

        StringTokenizer st = new StringTokenizer(init_string);

        int token_count = st.countTokens();
        if(token_count < 2) {
            logger.severe("AFT Vsites initialisation line is incorrect for <" +vsite_name +"> expects at least 2 words in this part <"+init_string+">");
            return false;
        }

        // Extract initialisation data from the string
        String host_name = st.nextToken();
        String s_port = st.nextToken();

        // Get the IP address of the machine hosting the NJS
        try {
            host_address = InetAddress.getByName(host_name);
        }
        catch(UnknownHostException uhex) {
            logger.severe("Vsite initialisation. Unknown host <"+host_name+"> for Vsite <"+vsite_name+"> ");
            return false;
        }

        // convert the port string into a number
        try {
            host_port = (new Integer(s_port)).intValue();
        }
        catch(NumberFormatException nfex) {
            logger.severe("Vsite initialisation. Port number not an integer <"+s_port+"> for Vsite <"+vsite_name+"> ");
            return false;
        }

        logger.config("Vsite <"+vsite_name+"> will use <"+host_name +":" + s_port+">");

        return true;
    }

    public String toString() {
        return "AFT Vsite <"+vsite_name+"> at <"+host_address.getHostName()+":"+host_port+">";
    }

    /**
     * Get a connection to the Vsite
     *
     **/
    public VsiteConnection getConnection() throws Exception {
        return new MyVsiteConnections(host_address,host_port,vsite_name);
    }

    private class MyVsiteConnections implements VsiteConnection {

        private String vsite_name;

        private Socket to_njs;

        public MyVsiteConnections(InetAddress host_address, int host_port, String vsite_name) throws Exception {
            to_njs = new Socket(host_address,host_port);
            this.vsite_name=vsite_name;
        }

        public void processRequest(java.io.ObjectOutputStream oos,java.io.ObjectInputStream ois,java.net.Socket socket,org.unicore.upl.ServerRequest sr) throws Exception {


            ObjectOutputStream to_njs_oos = null;
            ObjectInputStream from_njs_ois = null;
            try {
                // ois has had the ServerRequest (sr) read off it, for now do not pass it on
                // (should really as the NJS should authorise this request). This means that we
                // must recreate the ObjectStream so that the NJS gets a new one

//040203 Don't bother typing (file to file sends three dirrerent Objects
//040203                org.unicore.AJOIdentifier uspace = (org.unicore.AJOIdentifier)ois.readObject();
//040203                org.unicore.PortfolioIdentifier portfolio = (org.unicore.PortfolioIdentifier)ois.readObject();
//040203                org.unicore.User user = (org.unicore.User) ois.readObject();

                // Write to the NJS
                to_njs_oos = new ObjectOutputStream(to_njs.getOutputStream());
                to_njs_oos.writeObject(ois.readObject());
                to_njs_oos.writeObject(ois.readObject());
                to_njs_oos.writeObject(ois.readObject());

//040203                to_njs_oos = new ObjectOutputStream(to_njs.getOutputStream());
//040203                to_njs_oos.writeObject(uspace);
//040203                to_njs_oos.writeObject(portfolio);
//040203                to_njs_oos.writeObject(user);

                to_njs_oos.flush();

                // ... and pass whatever we get back from the NJS to the client
                from_njs_ois = new ObjectInputStream(to_njs.getInputStream());
                Integer hdr = (Integer) from_njs_ois.readObject();
                oos.writeObject(hdr);
                if(hdr.intValue() == 0) {
                    oos.writeObject(from_njs_ois.readObject());
                    oos.writeObject(from_njs_ois.readObject());
                    oos.writeObject(from_njs_ois.readObject());
                }
                else {
                    oos.writeObject(from_njs_ois.readObject());
                }
            }
            finally {
                try {
                    to_njs_oos.close();
                    from_njs_ois.close();
                }
                catch(Exception ex) {
                }
            }

        }

        public void close() {
            try {
                to_njs.close();
            }
            catch(Exception ex) {
            }
        }

        public String getTarget() {
            return vsite_name;
        }
    }

}

//  Changes:
//  12-09-02 SvdB Created
//
//           Copyright (c) Fujitsu Ltd 2000 - 2002
//
// Use and distribution is subject to the Community Source Licence.
//
// A copy of the Community Source Licence came with this distribution.
// Copies are also available at http://www.fecit.co.uk/arcon or by
// email from arcon@fecit.co.uk.
